#include <iostream>

#include "services.h"
#include "contact-core.h"

#include "dummy-main.h"

using namespace Ekiga;

class ContactObserver
{
public:

  ContactObserver (std::tr1::shared_ptr<ContactCore> core_): core(core_)
  {
    core->visit_sources (sigc::bind_return (sigc::mem_fun (this, &ContactObserver::on_source_added), true));
    core->source_added.connect (sigc::mem_fun (this, &ContactObserver::on_source_added));
    core->book_added.connect (sigc::mem_fun (this, &ContactObserver::on_book_added));
    core->book_removed.connect (sigc::mem_fun (this, &ContactObserver::on_book_removed));
    core->book_updated.connect (sigc::mem_fun (this, &ContactObserver::on_book_updated));
    core->contact_added.connect (sigc::mem_fun (this, &ContactObserver::on_contact_added));
    core->contact_removed.connect (sigc::mem_fun (this, &ContactObserver::on_contact_removed));
    core->contact_updated.connect (sigc::mem_fun (this, &ContactObserver::on_contact_updated));
  }


private:

  std::tr1::shared_ptr<ContactCore> core;

  void on_source_added (std::tr1::shared_ptr<Source> source)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
    source->visit_books (sigc::mem_fun (this, &ContactObserver::visit_book));
  }

  bool visit_book (std::tr1::shared_ptr<Book> book)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
    book->visit_contacts (sigc::mem_fun (this, &ContactObserver::visit_contact));
    return true;
  }

  bool visit_contact (std::tr1::shared_ptr<Contact>)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
    return true;
  }
  
  void on_book_added (std::tr1::shared_ptr<Source>,
		      std::tr1::shared_ptr<Book> book)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
    book->visit_contacts (sigc::mem_fun (this, &ContactObserver::visit_contact));
  }


  void on_book_removed (std::tr1::shared_ptr<Source>,
			std::tr1::shared_ptr<Book>)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
  }

  void on_book_updated (std::tr1::shared_ptr<Source>,
			std::tr1::shared_ptr<Book>)
  {
  }

  void on_contact_added (std::tr1::shared_ptr<Source>,
			 std::tr1::shared_ptr<Book>,
			 std::tr1::shared_ptr<Contact>)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
  }

  void on_contact_removed (std::tr1::shared_ptr<Source>,
			   std::tr1::shared_ptr<Book>,
			   std::tr1::shared_ptr<Contact>)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
  }

  void on_contact_updated (std::tr1::shared_ptr<Source>,
			   std::tr1::shared_ptr<Book>,
			   std::tr1::shared_ptr<Contact>)
  {
    std::cout << __PRETTY_FUNCTION__ << std::endl;
  }
};

int
main (int argc,
      char* argv[])
{
  ServiceCore services;

  std::tr1::shared_ptr<Service> service (new BasicService ("name", "\tdescription"));
  std::tr1::shared_ptr<ContactCore> contacts (new ContactCore ());
  std::tr1::shared_ptr<ContactObserver> observer (new ContactObserver(contacts));

  services.add (service);
  services.add (contacts);

  dummy_init (services, &argc, &argv);

  return 0;
}
